#include "Device.h"
#include "lqmeas/log/Log.h"

namespace LQMeas {
    Device::Device(QObject *parent) :
        QObject(parent) {
        qRegisterMetaType<LQMeas::Error>("LQMeas::Error");
    }


    QString Device::devStr() const {
        QString ret = modificationName();
        if (!serial().isEmpty())
            ret.append(", ").append(serial());
        return ret;
    }

    QString Device::devStrWithLocation() const {
        QString ret = devStr();
        if (!location().isEmpty())
            ret.append(" (").append(location()).append(")");
        return ret;
    }

    QString Device::modificationName() const {
        return name();
    }

    QString Device::ifaceName() const {
        DeviceInterface cur_iface = iface();
        if (cur_iface == IFACE_USB)
            return "USB";
        if (cur_iface == IFACE_PCI)
            return "PCI (PCIe)";
        if (iface() == IFACE_ETH)
            return "Ethernet";
        return "Unknown";
    }

    QString Device::devKey() const {
        return devStr() + ", " + ifaceName();
    }

    Error Device::open() {
        Error err = protOpen();
        if (err.isSuccess()) {
            LQMeasLog->detail(tr("Device opened successfully"), this);
        } else {
            LQMeasLog->error(tr("Device open error"), err, this);
        }
        return err;
    }

    Error Device::close() {
        Error err = protClose();
        if (err.isSuccess()) {
            LQMeasLog->detail(tr("Device closed successfully"), this);
        } else {
            LQMeasLog->error(tr("Device close error"), err, this);
        }
        return err;
    }

    Error Device::configure() {
        Error err = protConfigure();
        if (err.isSuccess()) {
            LQMeasLog->detail(tr("Device configuration done successfully"), this);
        } else {
            LQMeasLog->error(tr("Device configuration error"), err, this);
        }
        return err;
    }

    Error Device::loadConfig(QString key, QSettings &set) {
        Error err = Error::Success();        
        set.beginGroup(key);

        err = protLoadConfig(set);

        set.endGroup();
        if (err.isSuccess())
            err = configure();
        return err;
    }

    Error Device::saveConfig(QString key, QSettings &set) {
        Error err;        
        set.beginGroup(key);
        err = protSaveConfig(set);
        set.endGroup();
        return err;
    }

    void Device::sendOpProgess(OP_STATUS status, QString descr, int done, int size, Error err) {
        emit opProgress(status, descr, done, size, err);
    }
}
