#ifndef LQMEAS_DEVICE_H
#define LQMEAS_DEVICE_H

#include <QObject>
#include <QSettings>

#include "lqmeas/StdErrors.h"

namespace LQMeas {

    class DevicesResolver;
    class DevOutStream;
    class DevOut;
    class DevAdc;

    class Device : public QObject {
        Q_OBJECT
    public:

        enum OP_STATUS {
            OP_START,
            OP_FINISH,
            OP_ERROR,
            OP_PROGR
        };

        enum DeviceInterface {
            IFACE_UNKNOWN = 0,
            IFACE_USB     = 1,
            IFACE_PCI     = 2,
            IFACE_ETH     = 3
        };


        virtual ~Device() {}

        virtual QString name() const = 0;
        virtual QString serial() const = 0;
        virtual QString location() const {return QString();}
        virtual QString devStr() const;
        virtual QString devStrWithLocation() const;
        virtual QString modificationName() const;

        virtual bool isOpened() const = 0;

        virtual DeviceInterface iface() const {return IFACE_UNKNOWN;}
        QString ifaceName() const;
        virtual QString devKey() const;


        virtual DevicesResolver *childResolver() {return 0;}

        virtual DevOutStream *devOutStream() {return 0;}
        virtual DevOut *devOut() {return 0;}
        virtual DevAdc *devAdc() {return 0;}

    signals:
        void opProgress(Device::OP_STATUS status, QString descr, int done, int size, Error err);
        void devDescrChanged();
    public slots:
        Error open();
        Error close();
        Error configure();
        Error loadConfig(QString key, QSettings &set);
        Error saveConfig(QString key, QSettings &set);

    protected:
        virtual Error protOpen()=0;
        virtual Error protClose()=0;
        virtual Error protLoadConfig(QSettings &set) = 0;
        virtual Error protSaveConfig(QSettings &set) const = 0;
        virtual Error protConfigure() = 0;

        void sendOpProgess(OP_STATUS status, QString descr, int done, int size, Error err);

        explicit Device(QObject *parent = 0);


        friend class DevAdc;
        friend class DevOutStream;
    };
}
#endif // LQMEAS_DEVICE_H
