#include "lcompDevice.h"
#include "lcomp_headers.h"




QString lcompDevice::firmwareDir() {
    return "firmware";
}

lcompDevice::~lcompDevice() {
    m_dev->Release();
}

LQMeasError lcompDevice::protOpen() {
    HANDLE hVxd = m_dev->OpenLDevice(); // открываем устройство
    LQMeasError err = LQMeasError::Success();

    if(hVxd == INVALID_HANDLE_VALUE) {
        err = lcompErrors::DeviceOpen();
    } else {
        QString fw_name = firmwareName();
        if (!fw_name.isEmpty()) {
            fw_name = firmwareDir() + "/" + fw_name;
            if (m_dev->LoadBios(fw_name.toAscii().data())!=L_SUCCESS) {
                err = lcompErrors::LoadBios();
            }
        }
    }

    if (err.isSuccess() && (m_dev->PlataTest()!=L_SUCCESS)) {
        err = lcompErrors::BoardTest();
    }

    if (err.isSuccess()) {
        PLATA_DESCR_U2 pd;
        if (m_dev->ReadPlataDescr(&pd)!=L_SUCCESS) {
            err = lcompErrors::ReadBoardDescr();
        } else {
            err = protFillDescr(&pd);
        }
    }

    if (!err.isSuccess()) {
        protClose();
    } else {
        m_is_opened = true;
    }

    return err;
}

LQMeasError lcompDevice::protClose() {
    LQMeasError err = LQMeasError::Success();
    if (m_dev->CloseLDevice()==L_SUCCESS) {
        m_is_opened = false;
    } else {
        err = lcompErrors::DeviceClose();
    }
    return err;
}

LQMeasError lcompDevice::lcompDeviceStart() {
    LQMeasError err = LQMeasError::Success();
    if (m_dev->InitStartLDevice()!=L_SUCCESS) {
        err = lcompErrors::AdcStartInit();
    } else if (m_dev->StartLDevice()!=L_SUCCESS) {
        err = lcompErrors::AdcStart();
    }
    return err;
}

lcompDevice::lcompDevice(IDaqLDevice *dev, const __SLOT_PARAM *par, unsigned slot)
    : LQMeasDevice(), m_slot(slot), m_board_type(par->BoardType), m_dev(dev), m_is_opened(false) {

}
