#include "lcompResolver.h"
#include "lcomp_headers.h"

#include "lqmeas_config.h"

#ifdef LQMEAS_DEV_E2010
    #include "e2010.h"
#endif
#ifdef LQMEAS_DEV_E14_440
    #include "e14_440.h"
#endif

#include <QLibrary>

lcompResolver::lcompResolver() : m_create_func(NULL) {
}


lcompResolver *lcompResolver::resolver() {
    static lcompResolver resolver;
    return &resolver;
}

QList<QSharedPointer<LQMeasDevice> > lcompResolver::getDevList() {
    if (!m_create_func) {
        QLibrary lib(sizeof(void*)==8 ? "lcomp64" : "lcomp");
        m_create_func = (CREATEFUNCPTR)lib.resolve("CreateInstance");
        if (m_create_func==NULL) {
            qxtLog->error(QObject::tr("Cannot load lcomp dynamic library"));
        }
    }

    if (m_create_func != NULL) {
        for (unsigned slot=0; slot < 255; slot++) {
            LUnknown* pIUnknown = m_create_func(slot);
            if (pIUnknown) {
                IDaqLDevice* pI;
                HRESULT hr = pIUnknown->QueryInterface(IID_ILDEV,(void**)&pI);
                if(SUCCEEDED(hr)) {
                    HANDLE hVxd;
                    pIUnknown->Release();
                    hVxd = pI->OpenLDevice(); // открываем устройство
                    if(hVxd != INVALID_HANDLE_VALUE) {
                        SLOT_PAR sl;
                        ULONG status = pI->GetSlotParam(&sl);
                        pI->CloseLDevice();
                        if(status == L_SUCCESS) {
                            QSharedPointer<LQMeasDevice> dev;
                            switch (sl.BoardType) {
#ifdef LQMEAS_DEV_E2010
                                case E2010:
                                case E2010B:
                                    dev = QSharedPointer<LQMeasDevice>(new e2010(&sl, slot));
                                    break;

#endif
#ifdef LQMEAS_DEV_E14_440
                                case E440:
                                    dev = QSharedPointer<LQMeasDevice>(new e14_440(pI, &sl, slot));
                                    break;
#endif
                                default:
                                    pI->Release();
                                    break;
                            }

                            if (dev) {
                                m_devs[slot] = dev;
                            } else {
                                m_devs.remove(slot);
                            }
                        }
                    }
                }
            } else {
                if ((GetLastError()!=L_ERROR_INUSE) && m_devs.contains(slot)) {
                    m_devs.remove(slot);
                }
            }
        }
    }
    return m_devs.values();
}
