#ifndef LCOMPSTREAMDEVICE_H
#define LCOMPSTREAMDEVICE_H


#include "lcompDevice.h"
#include "lcompStreamReceiver.h"
#include "../../ifaces/adc/LQMeasAdc.h"
#include "lcompRecvProcessorIface.h"

#include <QThread>

#define LCOMP_SEQ_COM_CH_CNT   32
#define LCOMP_SEQ_DIFF_CH_CNT  16




class lcompStreamDevice : public lcompDevice, public LQMeasAdc, public lcompRecvProcessorIface {
    Q_OBJECT
public:


    struct AdcConfig {
        enum ADC_MODE {
            ADC_MODE_DIFF = 0,
            ADC_MODE_COMM = 1,
            ADC_MODE_ZERO = 2
        };

        struct {
            bool Enabled;
            ADC_MODE Mode;
            unsigned Range;
        } Ch[LCOMP_SEQ_COM_CH_CNT];

        double FreqAdc;
        double FreqCh;
    } ;



    double adcFreq() const;
    double adcChFreq() const;
    bool adcChEnabled(int ch) const;
    double adcChRange(int ch) const;

    AdcConfig adcCurConfig() const {return m_adcCfg;}
    LQMeasError adcSetConfig(AdcConfig cfg);


    LQMeasError adcGetData(double *data, unsigned size, unsigned flags,
                           unsigned tout, unsigned *recvd_size);


protected:
    lcompStreamDevice(IDaqLDevice *dev, const __SLOT_PARAM *par, unsigned slot,
                      LQMeasAdc::AdcType adcType);

    LQMeasError protAdcStart();
    LQMeasError protAdcStop();
    LQMeasError protAdcSetRawConfig(const void *rawCfg, size_t cfg_size);

    LQMeasError protLoadConfig(QSettings &set);
    LQMeasError protSaveConfig(QSettings &set) const;

    AdcConfig m_adcCfg;
private:

    LQMeasError fillAdcParams(void *par);

    LQMeasError lcompProcessRecvdData(const void *src_buf, unsigned size, double *dst_buf, unsigned flags, unsigned offs);

    lcompStreamReceiver m_adcReceiver;

};

#endif // LCOMPSTREAMDEVICE_H
