#include "LtrCrate.h"
#include <QScopedPointer>
#include "../modules/LtrModule.h"
#include "../LtrResolver.h"
#include "lqmeas/lqtdefs.h"


LtrCrate::LtrCrate(TLTR hnd, QString typeName, QString serial, unsigned slot_cnt) :
    m_typeName(typeName), m_serial(serial), m_slot_cnt(slot_cnt), m_hnd(hnd) {

    m_modules.resize(slot_cnt);
}

LtrCrate::~LtrCrate() {
    if (isOpened())
        close();
}

bool LtrCrate::isOpened() const {
    return LTR_IsOpened(&m_hnd)==LTR_OK;
}



quint32 LtrCrate::ltrdIpAddr() {
    return m_hnd.saddr;
}

quint16 LtrCrate::ltrdTcpPort() {
    return m_hnd.sport;
}

QList<QSharedPointer<LtrModule> > LtrCrate::getModuleList(QStringList nameFilter) {
    QScopedPointer<LQMeasDeviceValidator> val(new LQMeasDeviceNameValidator(nameFilter));
    return getModuleList(val.data());
}



QList<QSharedPointer<LtrModule> > LtrCrate::getModuleList(const LQMeasDeviceValidator *validator) {
    QList<QSharedPointer<LtrModule> > ret;
    bool close_req = false;
    if (!isOpened()) {
        if (open().isSuccess()) {
            close_req = true;
        }
    }

    if (isOpened()) {
        WORD mids[LTR_MODULES_PER_CRATE_MAX];
        int err = LTR_GetCrateModules(&m_hnd, mids);
        if (err==LTR_OK) {
            for (int slot=0; (slot < LTR_MODULES_PER_CRATE_MAX) && (slot < m_modules.size()); slot++) {
                if ((mids[slot]!=LTR_MID_EMPTY) && (mids[slot]!=LTR_MID_IDENTIFYING)) {
                    QSharedPointer<LtrModule> module = m_modules[slot];
                    if (!module || (module->mid()!= mids[slot])) {
                        m_modules[slot] = module = LtrResolver::createModule(
                                    LtrResolver::resolver()->getCrate(m_serial),
                                    slot+LTR_CC_CHNUM_MODULE1, mids[slot]);
                    }

                    if (module && ((validator == 0) || validator->deviceValid(module))) {
                        ret.append(module);
                    }

                } else {
                    /* если пуст - то удаляем старую информацию о модуле */
                    m_modules[slot] = QSharedPointer<LtrModule>();
                }
            }
        }


        if (close_req)
            close();
    }
    return ret;
}

QList<QSharedPointer<LQMeasDevice> > LtrCrate::getDevList() {
    QList<QSharedPointer<LtrModule> > ltrlist = getModuleList(0);
    QList<QSharedPointer<LQMeasDevice> > retlist;
    foreach (QSharedPointer<LtrModule> ltr, ltrlist) {
        retlist.append(ltr);
    }
    return retlist;
}

LQMeasError LtrCrate::error(int err) const {
    return err == LTR_OK ? LQMeasError::Success() :
                           LQMeasError(err, QSTRING_FROM_CSTR(LTR_GetErrorString(err)),
                                       LQMeasError::ERRTYPE_DEVNATIVE_LTR);
}


LQMeasError LtrCrate::protOpen() {
    return error(LTR_Open(&m_hnd));
}

LQMeasError LtrCrate::protClose() {
    return error(LTR_Close(&m_hnd));
}

LQMeasError LtrCrate::protLoadConfig(QSettings &set) {
    return error(LTR_ERROR_NOT_IMPLEMENTED);
}

LQMeasError LtrCrate::protSaveConfig(QSettings &set) const {
    return error(LTR_ERROR_NOT_IMPLEMENTED);
}

LQMeasError LtrCrate::protConfigure() {
    return LQMeasError::Success();
}
