#ifndef LTRCRATE_H
#define LTRCRATE_H

#include <QObject>
#include <QStringList>
#include <QSharedPointer>
#include <QVector>
#include "ltr/include/ltrapi.h"

#include "lqmeas/devs/LQMeasDeviceValidator.h"
#include "lqmeas/devs/resolver/LQDevResolver.h"

class LtrModule;
class LtrResolver;


class LtrCrate : public LQMeasDevice, public LQDevResolver {
    Q_OBJECT
public:    
    ~LtrCrate();

    QString name() const {return m_typeName;}
    QString serial() const {return m_serial;}
    unsigned slotCount() const {return m_slot_cnt;}


    virtual bool isOpened() const;
    /* адрес ltrd с которым установлено соединение */
    quint32 ltrdIpAddr();
    /* порт TCP соединения с ltrd */
    quint16 ltrdTcpPort();




    QList<QSharedPointer<LtrModule> > getModuleList(QStringList nameFilter);
    QList<QSharedPointer<LtrModule> > getModuleList(const LQMeasDeviceValidator* validator);
    QList<QSharedPointer<LQMeasDevice> > getDevList();

    LQDevResolver *childResolver() {return dynamic_cast<LQDevResolver *>(this);}


    LQMeasError error(int err) const;
protected:
    LQMeasError protOpen();
    LQMeasError protClose();
    LQMeasError protLoadConfig(QSettings &set);
    LQMeasError protSaveConfig(QSettings &set) const;
    LQMeasError protConfigure();


private:
    explicit LtrCrate(TLTR hnd, QString typeName, QString serial, unsigned slot_cnt);


    QString m_typeName;
    QString m_serial;
    unsigned m_slot_cnt;

    QVector<QSharedPointer<LtrModule> > m_modules;

    mutable TLTR m_hnd;

    friend class LtrResolver;
};

#endif // LTRCRATE_H
