#include "LtrModule.h"
#include "lqmeas/lqtdefs.h"

#include <QCoreApplication>


LtrModule::LtrModule(QSharedPointer<LtrCrate> crate, unsigned slot, unsigned mid, QObject *parent) :
    LQMeasDevice(parent), m_crate(crate), m_slot(slot), m_mid(mid) {
}

LtrModule::~LtrModule() {

}

unsigned LtrModule::mid() const {
    return m_mid;
}

int LtrModule::slot() const {
    return m_slot;
}

LQMeasError LtrModule::error(int err) const {
    return err == LTR_OK ? LQMeasError::Success() : LQMeasError(err, errorString(err),
                                                                LQMeasError::ERRTYPE_DEVNATIVE_LTR);
}

QString LtrModule::errorString(int err) const {
    return  QSTRING_FROM_CSTR(LTR_GetErrorString(err));
}

LQMeasError LtrModule::rawWordsReceive(DWORD *wrds, unsigned size, unsigned tout, unsigned *recvd_size) {
    LQMeasError err = LQMeasError::Success();
    INT recv_res = LTR_Recv(channel(), wrds, 0, size, tout);
    if (recv_res >= 0) {
        *recvd_size = recv_res;
    } else {
        err = error(recv_res);
    }
    return err;
}

LQMeasError LtrModule::rawWordsSend(const DWORD *wrds, unsigned size, unsigned tout, unsigned *sent_size) {
    LQMeasError err = LQMeasError::Success();
    int send_res = LTR_Send(channel(), wrds, size, tout);
    if (send_res >= 0) {
        *sent_size = send_res;
    } else {
        err = error(send_res);
    }
    return err;
}


