#ifndef LTR210_H
#define LTR210_H

#include "LtrModule.h"
#include "ltr/include/ltr210api.h"
#include "../../../ifaces/adc/LQMeasAdc.h"

class LTR210 : public LtrModule, public LQMeasAdc {
    Q_OBJECT
public:
    static QString typeModuleName();
    static const quint16 typeModuleID = LTR_MID_LTR210;
    static const unsigned typeAdcChCnt = LTR210_CHANNEL_CNT;


    ~LTR210();

    QString name() const;
    QString serial() const;
    unsigned verFPGA() const;
    unsigned verPLD() const;


    double adcFreq() const;
    double adcChRange(int ch_num) const;

    const double adcRangeMaxVal(int range) const;
    const double adcRangeMinVal(int range) const;
    unsigned adcRangesCnt() const;
    double adcFreqMax() const;

    QString errorString(int err) const;
    bool isOpened() const;

    bool adcIsRunning() const;
    bool adcChEnabled(int ch) const;

signals:

public slots:
    LQMeasError startFrame();
    LQMeasError measZeroAllRanges();
    LQMeasError getFrame(double *data, unsigned size, unsigned flags, unsigned tout);
    LQMeasError waitSof(unsigned tout);


    LQMeasError adcGetData(double *data, unsigned size, unsigned flags, unsigned tout,
                   unsigned *recvd_size, TLTR210_FRAME_STATUS *status);

    LQMeasError loadFPGA(QString filename);



    LQMeasError adcGetData(double *data, unsigned size, unsigned flags,
                           unsigned tout, unsigned *recvd_size);

protected:
    virtual TLTR* channel() const;

    LQMeasError protOpen();
    LQMeasError protClose();
    LQMeasError protConfigure();
    LQMeasError protLoadConfig(QSettings &set);
    LQMeasError protSaveConfig(QSettings &set) const;
    LQMeasError protAdcSetRawConfig(const void *cfg, size_t cfg_size);
    
    LQMeasError protAdcStart();
    LQMeasError protAdcStop();
private:
    explicit LTR210(QSharedPointer<LtrCrate> crate, unsigned slot, QObject *parent = 0);

    static void APIENTRY cb_load_porgr(void* cb, TLTR210 *hnd, DWORD done_cnt, DWORD total_cnt);
    static QString loadFpgaOpDescr() {return tr("Загрузка прошивки ПЛИС");}

    mutable TLTR210 m_hnd;

    friend class LtrResolver;
    double m_zeroOffs[typeAdcChCnt][LTR210_RANGE_CNT];

};

#endif // LTR210_H
