#ifndef LTR24_H
#define LTR24_H

#include "LtrModule.h"
#include "ltr/include/ltr24api.h"
#include "../../../ifaces/adc/LQMeasAdc.h"
#include "../../LQMeasDevFrameReceiver.h"



class LTR24 : public LtrModule, public LQMeasAdc {
    Q_OBJECT
public:
    static QString typeModuleName();
    static const quint16 typeModuleID = LTR_MID_LTR24;

    struct AdcConfig {
        BYTE        ADCFreqCode;
        BYTE        DataFmt;
        BYTE        ISrcValue;
        BOOL        TestMode;
        struct {
            BOOL    Enable;
            BYTE    Range;
            BOOL    AC;
            BOOL    ICPMode;
        } ChannelMode[LTR24_CHANNEL_NUM];
    };


    QString serial() const;
    QString name() const;
    QString modificationName() const;
    int pldVer() const;
    bool supportICP() const;

    const double adcRangeMaxVal(int range) const;
    const double adcRangeMinVal(int range) const;
    unsigned adcRangesCnt() const;
    double adcFreqMax() const;

    static double typeAdcFreqVal(int idx);
    static unsigned typeAdcFreqsCnt();
    double adcFreqVal(int idx) const {return typeAdcFreqVal(idx); }
    unsigned adcFreqsCnt() const {return typeAdcFreqsCnt(); }


    double adcFreq() const;
    double adcChRange(int ch) const;

    QString errorString(int err) const;
    bool isOpened() const;

        
    bool adcIsRunning() const;
    bool adcChEnabled(int ch) const;

signals:
    
public slots:

    LQMeasError adcGetData(double *data, unsigned size, unsigned flags,
                           unsigned tout, unsigned *recvd_size);
protected:
    TLTR* channel() const;

    LQMeasError protOpen();
    LQMeasError protClose();
    LQMeasError protConfigure();
    LQMeasError protLoadConfig(QSettings &set);
    LQMeasError protSaveConfig(QSettings &set) const;

    LQMeasError protAdcSetRawConfig(const void *rawCfg, size_t cfg_size);
    LQMeasError rawWordsReceive(DWORD *wrds, unsigned size, unsigned tout, unsigned *recvd_size);

    LQMeasError protAdcStart();
    LQMeasError protAdcStop();


private:
    explicit LTR24(QSharedPointer<LtrCrate> crate, unsigned slot, QObject *parent = 0);
    mutable TLTR24 m_hnd;

    LQMeasDevFrameReceiver<LTR24, DWORD> m_receiver;

    friend class LQMeasDevFrameReceiver<LTR24, DWORD>;
    friend class LtrResolver;

};

#endif // LTR24_H
