#ifndef LTR25_H
#define LTR25_H

#include "LtrModule.h"
#include "ltr/include/ltr25api.h"
#include "lqmeas/ifaces/adc/LQMeasAdc.h"
#include "lqmeas/devs/LQMeasDevFrameReceiver.h"

class LTR25 : public LtrModule, public LQMeasAdc {
    Q_OBJECT
public:
    static QString typeModuleName();
    static const quint16 typeModuleID = LTR_MID_LTR25;

    QString name() const;
    QString serial() const;
    unsigned verFPGA() const;
    unsigned verPLD() const;
    unsigned boardRevision() const;
    bool industrial() const;


    const double adcRangeMaxVal(int range) const;
    const double adcRangeMinVal(int range) const;
    unsigned adcRangesCnt() const;
    double adcFreqMax() const;


    static double typeAdcFreqVal(int idx);
    static unsigned typeAdcFreqsCnt();
    double adcFreqVal(int idx) const {return typeAdcFreqVal(idx); }
    unsigned adcFreqsCnt() const {return typeAdcFreqsCnt(); }


    double adcFreq() const;
    double adcChRange(int ch) const;

    QString errorString(int err) const;
    bool isOpened() const;

    bool adcIsRunning() const;
    bool adcChEnabled(int ch) const;


    void *rawHandle() {return &m_hnd;}


signals:

public slots:


    LQMeasError adcGetData(double *data, unsigned size, unsigned flags,
                   unsigned tout, unsigned *recvd_size);
protected:
    TLTR* channel() const;

    LQMeasError protOpen();
    LQMeasError protClose();
    LQMeasError protConfigure();
    LQMeasError protLoadConfig(QSettings &set);
    LQMeasError protSaveConfig(QSettings &set) const;
    LQMeasError protAdcSetRawConfig(const void *cfg, size_t cfg_size);

    LQMeasError rawWordsReceive(DWORD *wrds, unsigned size, unsigned tout, unsigned *recvd_size);

    LQMeasError protAdcStart();
    LQMeasError protAdcStop();
private:
    explicit LTR25(QSharedPointer<LtrCrate> crate, unsigned slot, QObject *parent = 0);

    mutable TLTR25 m_hnd;

    LQMeasDevFrameReceiver<LTR25, DWORD> m_receiver;

    friend class LQMeasDevFrameReceiver<LTR25, DWORD>;
    friend class LtrResolver;
};

#endif // LTR25_H
