#ifndef LTR35_H
#define LTR35_H

#include "LtrModule.h"
#include "lqmeas/ifaces/dac/LQMeasOutStream.h"
#include "lqmeas/devs/LQMeasDevFrameSender.h"

#include "ltr/include/ltr35api.h"


class LTR35 : public LtrModule, public LQMeasOutStream {
    Q_OBJECT
public:    
    static QString typeModuleName();
    static const quint16 typeModuleID = LTR_MID_LTR35;


    ~LTR35();

    QString name() const;
    QString modificationName() const;
    QString serial() const;
    unsigned verFPGA() const;
    unsigned verPLD() const;

    unsigned modification();


    QString errorString(int err) const;
    bool isOpened() const;

    LQMeasError outSignalsGenerate();

    double outDacMaxVal(int ch) const;
    double outDacMinVal(int ch) const;    
    double outDacMaxFreq() const;
    double outFreq() const {return m_outFreq;}



    void setRange(int ch, int range);
    int range(int ch);
    void setOutFreq(double freq) {m_outFreq = freq;}
    double outDacMaxVal() const;
    double dacMaxRangeVal(unsigned range) const;

    int outDacSetChMode(unsigned ch, OutChMode mode) {
        m_dacModes[ch] = mode;
        return 0;
    }

    OutChMode outDacChMode(unsigned ch) const {return m_dacModes[ch];}
    OutChMode outDigChMode(unsigned ch) const {return OutChModeRam; }

    LQMeasError outStopGenerate(unsigned tout = 5000) const;
signals:


protected:
    virtual TLTR* channel() const;

    LQMeasError protOpen();
    LQMeasError protClose();
    LQMeasError protConfigure();
    LQMeasError protLoadConfig(QSettings &set);
    LQMeasError protSaveConfig(QSettings &set) const;


    LQMeasError protOutStreamInit();
    LQMeasError protOutStreamStart();
    LQMeasError protOutStreamStop(unsigned tout);

    LQMeasError rawWordsSend(const DWORD *wrds, unsigned size, unsigned tout, unsigned *sent_size);
    LQMeasError protSendStreamData(const double *dac_data, unsigned dac_size,
                           const unsigned *dig_data, unsigned dig_size, unsigned flags, unsigned tout);
private:
    explicit LTR35(QSharedPointer<LtrCrate> crate, unsigned slot, QObject *parent = 0);
    double m_outFreq;
    OutChMode m_dacModes[LTR35_DAC_CHANNEL_CNT];

    mutable TLTR35 m_hnd;

    LQMeasDevFrameSender<LTR35, DWORD> m_sender;

    friend class LQMeasDevFrameSender<LTR35, DWORD>;
    friend class LtrResolver;
};

#endif // LTR35_H
