#ifndef LQMEAS_DEVADC_H
#define LQMEAS_DEVADC_H

#include <QVector>
#include "lqmeas/StdErrors.h"
#include "lqmeas/devs/Device.h"

namespace LQMeas {
    class DevAdc {
    public:

        enum ChannelStatus {
            ChStatusOk,
            ChStatusNoData,
            ChStatusDisabled,
            ChStatusOpen,
            ChStatusShort,
            ChStatusOverload,
            ChStatusError
        };

        enum AdcType {
            TypeParallel,   /** Параллельные каналы */
            TypeSequential  /** Последовательный опрос каналов (с коммутатором) */
        };


        virtual unsigned adcChannelsCnt() const;
        virtual const double adcRangeMaxVal(int range) const = 0;
        virtual const double adcRangeMinVal(int range) const = 0;
        virtual unsigned adcRangesCnt() const = 0;
        virtual double adcFreqMax() const = 0;

        virtual double adcFreq() const = 0;
        virtual double adcChFreq() const;


        virtual bool adcIsRunning() const {return m_run;}
        virtual bool adcChEnabled(int ch) const = 0;
        virtual bool adcExternalStart() const {return false;}

        virtual unsigned  adcEnabledChCnt() const;


        virtual Error adcEnable();
        virtual Error adcDisable();

        Error adcStart();
        Error adcStop();
        virtual Error adcSetRawConfig(const void* cfg, size_t cfg_size);
        virtual Error adcGetData(double *data, unsigned size, unsigned flags,
                               unsigned tout, unsigned *recvd_size) = 0;

        /* получить номер физ. канала по индексу при приеме */
        virtual int adcChNum(int idx) const;
        virtual int adcChIdx(int ch) const;

        virtual double adcChRange(int ch) const = 0;

        virtual ChannelStatus adcGetChStatus(int ch, bool clear = false);
    protected:
        explicit DevAdc(Device *dev, AdcType type = TypeParallel);


        virtual Error protAdcStart() = 0;
        virtual Error protAdcStop() = 0;
        virtual Error protAdcSetRawConfig(const void *rawCfg, size_t cfg_size)=0;

        void adcConfigDone(int err);
        void adcSetChCnt(int ch_cnt);
        void adcSetChStatus(int ch, ChannelStatus status);

        QVector<ChannelStatus> m_ch_status;
        QVector<int> m_ch_indexes;
        AdcType m_type;
        bool m_run;
        Device *m_dev;
    };
}
#endif // LQMEAS_DEVADC_H
