#ifndef LQMEAS_DEVOUT_H
#define LQMEAS_DEVOUT_H

#include "DacSignals/DacSignal.h"
#include "DigSignals/DigSignal.h"
#include <QVector>
#include <QSharedPointer>
#include <QSettings>
#include "lqmeas/StdErrors.h"
#include "lqmeas/units/Units.h"

namespace LQMeas {
    class Device;

    class DevOut {
    public:
        enum OutChMode {
            OutChModeRam = 0, /** сигнал генерируется по точкам в программно */
            OutChModeHard = 1 /** сигнал генерируется аппаратно по параметрам */
        };

        enum OutRamSyncMode {
            OutRamSyncModeCycle = 0,
            OutRamSyncModeStream = 1,
        };


        int outDacChannelsCnt() const;
        int outDigChannelsCnt() const;

        virtual double outFreq() const = 0;
        virtual bool outGenRunning() const = 0;

        Error outSignalsGenSetup();
        Error outSignalsGenStart();
        Error outSignalsGenStop();

        Error outSignalsGenerate();


        virtual Unit   outDacUnit(int ch) const {return Units::V();}
        virtual double outDacMaxVal(int ch) const = 0;
        virtual double outDacMinVal(int ch) const = 0;
        virtual double outDacMaxFreq() const = 0;


        virtual void outClearSignals();

        virtual Error outDacSetSignal(int ch, QSharedPointer<DacSignal> signal);
        QSharedPointer<DacSignal> outDacSignal(int ch) const;


        virtual Error outDigSetSignal(int ch, QSharedPointer<DigSignal> signal);
        QSharedPointer<DigSignal> outDigSignal(int ch) const;


        void outGenRamInit();

        Error outGenRamData(unsigned points, QVector<double> &dac_data,
                            QVector<unsigned> &dig_data);

        virtual bool outDacValidRamSignal(int ch) const;
        virtual bool outDigValidRamSignal(int ch) const;

        virtual OutChMode outDacChMode(int ch) const = 0;
        virtual OutChMode outDigChMode(int ch) const = 0;

        bool outDacChEnabled(int ch) const;
        bool outDigChEnabled(int ch) const;

        void outDacChEnable(int ch, bool en);
        void outDigChEnable(int ch, bool en);

        int outDacEnabledChCnt() const;
        int outDigEnabledChCnt() const;


        OutRamSyncMode outRamSyncMode() const {return m_outRamSyncMode;}
        void outSetRamSyncMode(OutRamSyncMode mode) {m_outRamSyncMode = mode;}

        Error outLoadSignalConfig(QSettings &set);
        Error outSaveSignalConfig(QSettings &set) const;

        virtual unsigned outGenSignalMaxSize() {return 0; }

    protected:
        DevOut(Device *dev);

        virtual Error protOutCycleLoadStart(unsigned size) = 0;
        virtual Error protOutCycleLoadFinish() = 0;
        virtual Error protOutCycleGenStart() = 0;
        virtual Error protOutCycleGenStop() = 0;
        virtual Error protOutSendData(const double *dac_data, unsigned dac_size,
                                      const unsigned *dig_data, unsigned dig_size,
                                      unsigned flags, unsigned tout) = 0;
        virtual Error protOutFlushData(unsigned tout) = 0;

        virtual unsigned outSendTout(unsigned dac_size, unsigned dout_size);


        void outSetDacChCnt(int ch_cnt);
        void outSetDigChCnt(int ch_cnt);

        Error outCalcCycleSigsParams(double out_freq, unsigned max_sig_size, unsigned *res_size);


    private:
        OutRamSyncMode m_outRamSyncMode;
        Device *m_dev;

        QVector< bool > m_dacChEnabled;
        QVector< bool > m_doutChEnabled;
        QVector< QSharedPointer<DacSignal> > m_sigsAnalog;
        QVector< QSharedPointer<DigSignal> > m_sigsDig;

        const double m_cycle_freq_rel_err;

    };
}

#endif // LQMEAS_DEVOUT_H
