#include "DevOutStream.h"
#include "lqmeas/devs/Device.h"
#include "lqmeas/log/Log.h"

namespace LQMeas {
    Error DevOutStream::outStreamInit(unsigned preload_points) {
        Error err = protOutStreamInit();
        if (err.isSuccess()) {
            QVector<double> dac_data;
            QVector<unsigned> dig_data;

            outSetRamSyncMode(OutRamSyncModeStream);

            QList<QSharedPointer<OutSignal> > sigs;
            for (int ch =0; ch < outDacChannelsCnt(); ch++) {
                if (outDacValidRamSignal(ch)) {
                    sigs.append(outDacSignal(ch));
                }
            }
            for (int ch =0; ch < outDigChannelsCnt(); ch++) {
                if (outDigValidRamSignal(ch)) {
                    sigs.append(outDigSignal(ch));
                }
            }

            double out_freq = outFreq();

            foreach (QSharedPointer<OutSignal> sig, sigs) {
                if (sig->signalFreq()) {
                    if (sig->periodStrictPoints()) {
                        unsigned pts = out_freq/sig->signalFreq() + 0.5;
                        if (pts < sig->minPoints())
                            pts = sig->minPoints();
                        sig->setRealParams(out_freq/pts, out_freq);
                    } else {
                        sig->setRealParams(sig->signalFreq(), out_freq);
                    }
                }
            }

            outGenRamInit();

            err = outGenRamData(preload_points, dac_data, dig_data);

            if (err.isSuccess()) {
                err = protOutSendData(dac_data.data(), dac_data.size(),
                                         dig_data.data(), dig_data.size(),
                                         0, 3000);
            }

            if (err.isSuccess()) {
                m_streamThread.init();
            }
        }
        return err;
    }

    Error DevOutStream::outStreamStart() {
        Error err = protOutStreamStart();
        if (err.isSuccess()) {
            m_streamThread.start();
            m_running = true;
            LQMeasLog->detail(Device::tr("Output data stream started successfully"), m_dev);
        } else {
            LQMeasLog->error(Device::tr("Output data stream start error"), err, m_dev);
        }
        return err;
    }

    Error DevOutStream::outStreamStop(unsigned tout) {
        Error err = Error::Success();
        if (m_running) {
            m_streamThread.setStopReq();
            if (!m_streamThread.wait(tout)) {
                err = StdErrors::StreamStopWait();
            } else {
                /** @todo учесть прошедшее время */
                err = protOutStreamStop(tout);
            }

            if (err.isSuccess()) {
                m_running = false;
                LQMeasLog->detail(Device::tr("Output data stream stopped successfully"), m_dev);
            } else {
                LQMeasLog->error(Device::tr("Output data stream stop error"), err, m_dev);
            }
        }
        return err;
    }

    DevOutStream::DevOutStream(Device *dev) : DevOut(dev), m_dev(dev), m_streamThread(this), m_running(false) {
        m_streamThread.moveToThread(&m_streamThread);

    }
}


