#ifndef LQMEAS_LOG_H
#define LQMEAS_LOG_H

#include <QObject>
#include <QHash>
#include "LogMessage.h"



#define LQMeasLog LQMeas::Log::instance()

namespace LQMeas {
    class Device;

    class Log : public QObject {
        Q_OBJECT
    public:
        static Log *instance();


        void retranslate();
        QList<LogLevel> logLevels() const;
        const LogLevel level(LogLevel::Number num) const;

    signals:
        void message(QSharedPointer<LogMessage> msg);
    public slots:


        void report(LogLevel::Number lvl, QString msg, Error err = Error::Success(),
                    Device *dev = 0);

        void info(QString msg, Device *dev = 0) {
            report(LogLevel::Info, msg, Error::Success(), dev);
        }
        void detail(QString msg, Device *dev = 0) {
            report(LogLevel::Detail, msg, Error::Success(), dev);
        }
        void error(QString msg, Error err, Device *dev = 0) {
            report(LogLevel::Error, msg, err, dev);
        }


    private:
        explicit Log();
        void initLevel(LogLevel::Number lvl, QColor col);

        QHash<LogLevel::Number, LogLevel> m_lvlsCfg;
    };
}

#endif // LQMEAS_LOG_H
