#ifndef MULTIMETERTHREAD_H
#define MULTIMETERTHREAD_H

#include <QThread>
#include <QVector>
#include <QMutex>
#include <QSharedPointer>
#include "lqmeas_config.h"

#include "../../devs/LQMeasDevice.h"
#include "../../ifaces/adc/LQMeasAdc.h"
class MultimeterProcessor;

class MultimeterThread : public QThread {
    Q_OBJECT
public:

    struct MultimeterChData {
        LQMeasAdc::ChannelStatus status;
        double DC;
#ifndef LQMEAS_SERVICE_MULTIMETER_DC_ONLY
        double AC;
        double freq;
        double SNR;
        double SINAD;
#endif
    };

    struct MultimeterData {
        int cfgID;
        QVector<MultimeterChData> ch;
    };

    explicit MultimeterThread(QObject *parent = 0);

    void setDev(QSharedPointer<LQMeasAdc> adc);



    void run();
    LQMeasError lastError() const;


signals:
    void dataReceived(MultimeterThread::MultimeterData data);
    void dataForProcess(LQMeasAdc *adc, QVector<double> vals, MultimeterThread::MultimeterData data);

public slots:
    void setConfig(void *cfg, size_t cfg_size, int meas_time, unsigned *id);
    void stopRequest();
private:
    bool restartReq() {return m_stopReq || m_cfgReq;}
    mutable QMutex m_lock;
    LQMeasError m_err;
    QSharedPointer<LQMeasDevice>  m_dev;
    LQMeasAdc *m_adc;
    unsigned char *m_curCfg;
    size_t m_cfgSize;

    int m_meas_time;
    bool m_stopReq;
    bool m_cfgReq;
    int m_cur_id;
    MultimeterProcessor *m_proc;
    QThread m_procThread;

};

#endif // MULTIMETERTHREAD_H
