#include "FreqConfigWidget.h"
#include <QHBoxLayout>
#include <QDoubleSpinBox>
#include <QComboBox>


FreqConfigWidget::FreqConfigWidget(QWidget *parent) : QWidget(parent) {
    QHBoxLayout *lout = new QHBoxLayout();
    m_freqEdit = new QDoubleSpinBox();
    m_freqEdit->setDecimals(3);

    lout->addWidget(m_freqEdit);

    m_unitBox = new QComboBox();
    m_unitBox->addItem(tr("Hz"), 1);
    m_unitBox->addItem(tr("KHz"), 1000);
    m_unitBox->addItem(tr("MHz"), 1000000);

    m_unitBox->setCurrentIndex(0);
    lout->addWidget(m_unitBox);
    lout->setMargin(0);
    setLayout(lout);

    connect(m_freqEdit, SIGNAL(editingFinished()), SIGNAL(editingFinished()));
    connect(m_unitBox, SIGNAL(currentIndexChanged(int)), SLOT(onUnitChanged(int)));
    connect(m_freqEdit, SIGNAL(valueChanged(double)), SLOT(onValueChanged(double)));
}

int FreqConfigWidget::unitMultCoef() const {
    return m_unitBox->itemData(m_unitBox->currentIndex()).toInt();
}

void FreqConfigWidget::loadUnitSettings(QSettings& set, QString key) {
    set.beginGroup(key);
    bool ok;
    int val = set.value("UnitMultCoef", 1).toInt(&ok);
    if (ok)
        setUnitMultCoef(val);
    set.endGroup();
}

void FreqConfigWidget::saveUnitSettings(QSettings& set, QString key) {
    set.beginGroup(key);
    set.setValue("UnitMultCoef", unitMultCoef());
    set.endGroup();
}

void FreqConfigWidget::setValue(double val) {
    m_val = val;
    m_freqEdit->setValue(val/unitMultCoef());
}

void FreqConfigWidget::setReadOnly(bool readOnly) {
    m_freqEdit->setReadOnly(readOnly);
    setUnitReadOnly(readOnly);
}

void FreqConfigWidget::setMaximum(double max) {
    m_max = max;
    m_freqEdit->setMaximum(m_max/unitMultCoef());
}

void FreqConfigWidget::setMinimum(double min) {
    m_min = min;
    m_freqEdit->setMinimum(m_min/unitMultCoef());
}

void FreqConfigWidget::setUnitReadOnly(bool readOnly) {
    m_unitBox->setEnabled(!readOnly);
}

void FreqConfigWidget::setUnitMultCoef(int mult) {
    for (int i=0; i < m_unitBox->count(); i++) {
        if (m_unitBox->itemData(i).toInt() == mult) {
            m_unitBox->setCurrentIndex(i);
            break;
        }
    }
}

void FreqConfigWidget::onUnitChanged(int idx) {
    int mult = unitMultCoef();
    disconnect(m_freqEdit, SIGNAL(valueChanged(double)), this, SLOT(onValueChanged(double)));
    m_freqEdit->setMaximum(m_max/mult);
    m_freqEdit->setMinimum(m_min/mult);
    m_freqEdit->setValue(m_val/mult);
    connect(m_freqEdit, SIGNAL(valueChanged(double)), this, SLOT(onValueChanged(double)));
    emit unitChanged(mult);
}

void FreqConfigWidget::onValueChanged(double val) {
    m_val = val * unitMultCoef();
}

