#include "DacSignalDialog.h"
#include "ui_DacSignalDialog.h"

#include "DacSignalParams/DacSignalParamUiSin.h"
#include "DacSignalParams/DacSignalParamUiConst.h"
#include "DacSignalParams/DacSignalParamUiPulse.h"
#include "DacSignalParams/DacSignalParamUiRamp.h"

DacSignalDialog::DacSignalDialog(QWidget *parent, LQMeas::DevOut *dac, int ch,
                                 QSharedPointer<LQMeas::DacSignal> signal) :
    QDialog(parent), m_dac(dac), m_ch(ch), m_curSigUi(0),
    m_curSignal(signal),
    ui(new Ui::DacSignalDialog) {

    ui->setupUi(this);
    int set_idx = -1;

    m_sigUi << new DacSignalParamUiConst() << new DacSignalParamUiSin() <<
               new DacSignalParamUiPulse() << new DacSignalParamUiRamp();


    m_const_item_cnt = ui->paramsLout->rowCount();

    foreach (DacSignalParamUi *paramUI, m_sigUi) {
        if (signal && (signal->typeName() == paramUI->typeName())) {
            paramUI->setSignal(signal);
            set_idx = ui->signalTypeBox->count();
        }
        ui->signalTypeBox->addItem(paramUI->showName(), paramUI->typeName());
    }

    if (set_idx < 0)
        set_idx = 0;

    ui->signalTypeBox->setCurrentIndex(set_idx);
    setUiByIdx(set_idx);

    connect(ui->signalTypeBox, SIGNAL(currentIndexChanged(int)), SLOT(setUiByIdx(int)));
}

DacSignalDialog::~DacSignalDialog() {
    qDeleteAll(m_sigUi);
    delete ui;
}

QSharedPointer<LQMeas::DacSignal> DacSignalDialog::getSignal() {
    return m_curSigUi ? m_curSigUi->createSignal() : QSharedPointer<LQMeas::DacSignal>();
}

void DacSignalDialog::setUiByIdx(int idx) {
    if (m_curSigUi) {
        m_curSigUi->freeUi();
        for (int pos = ui->paramsLout->rowCount()-1; pos >= m_const_item_cnt; pos--) {
            removeRow(pos);
        }
        m_curSigUi = 0;
    }

    if (idx >=0) {
        QString typeName = ui->signalTypeBox->itemData(idx).toString();
        foreach (DacSignalParamUi *paramUI, m_sigUi) {
            if (typeName == paramUI->typeName()) {
                m_curSigUi = paramUI;
                m_curSigUi->createUi(m_dac, m_ch, ui->paramsLout);
            }
        }
    }
}

void DacSignalDialog::removeRow(int row) {
    QFormLayout::ItemRole roles[] = {QFormLayout::FieldRole, QFormLayout::LabelRole, QFormLayout::SpanningRole};

    for (int role_ind = 0; role_ind < sizeof(roles)/sizeof(roles[0]); role_ind++) {
        QLayoutItem* wgtItem = ui->paramsLout->itemAt ( row, roles[role_ind] );
        if (wgtItem){
            QWidget *wgt = wgtItem->widget();
            ui->paramsLout->removeWidget(wgt);
            ui->paramsLout->removeItem(wgtItem);
            delete wgt;
        }
    }
}
