#include "OutSigGenaration.h"
#include "LQMeasStudio.h"
#include "lqmeas/ifaces/out/DacSignals/DacSignalSin.h"

    namespace LQMeasStudio {
    void OutSigGenaration::procStartPrepare() {
        foreach (QSharedPointer<LQMeas::Device> dev, LQMeasStudioEnv->deviceTree()->selectedList()) {
            LQMeas::DevOutStream *outStream = dev->devOutStream();
            if (outStream && (outStream->outRamSyncMode() == LQMeas::DevOut::OutRamSyncModeStream)) {
                if ((outStream->outDacEnabledChCnt() > 0) || (outStream->outDigEnabledChCnt() > 0)) {
                    LQMeas::Error err = outStream->outStreamInit(outStream->outFreq());
                    if (!err.isSuccess()) {
                        emit deviceError(dev, err, tr("Data setup error"), tr("Error occured during out stream initialization"));
                    } else {
                        m_outStreamDevs.append(outStream);
                        m_devs.append(dev);
                    }
                }
            } else {
                LQMeas::DevOut *devOut = dev->devOut();
                if (devOut && (outStream->outRamSyncMode() == LQMeas::DevOut::OutRamSyncModeCycle)) {
                    if ((devOut->outDacEnabledChCnt() > 0) || (devOut->outDigEnabledChCnt() > 0)) {
                        LQMeas::Error err = devOut->outSignalsGenSetup();
                        if (!err.isSuccess()) {
                            emit deviceError(dev, err, tr("Data setup error"), tr("Error occured during setup cycle signal data"));
                        } else {
                            m_outCycleDevs.append(devOut);
                            m_devs.append(dev);
                        }
                    }
                }
            }
        }
    }

    void OutSigGenaration::procStart() {
        foreach (LQMeas::DevOutStream *outStream, m_outStreamDevs) {
            connect(outStream->outStreamThread(), SIGNAL(finished()), SLOT(onThreadFinished()));
            LQMeas::Error err = outStream->outStreamStart();
            if (!err.isSuccess()) {
                sendError(outStream, err);
            }
        }

        foreach (LQMeas::DevOut *outCycle, m_outCycleDevs) {
            LQMeas::Error err =outCycle->outSignalsGenStart();
            if (!err.isSuccess()) {
                sendError(outCycle, err);
            }
        }
    }

    void OutSigGenaration::procStop() {
        foreach (LQMeas::DevOutStream *outStream, m_outStreamDevs) {
            if (outStream->outStreamIsRunning())
                outStream->outStreamStop();
            disconnect(outStream->outStreamThread(), SIGNAL(finished()), this, SLOT(onThreadFinished()));
        }
        foreach (LQMeas::DevOut *outCycle, m_outCycleDevs) {
            outCycle->outSignalsGenStop();
        }
    }

    void OutSigGenaration::procClear() {
        m_devs.clear();
        m_outStreamDevs.clear();
        m_outCycleDevs.clear();
    }

    int OutSigGenaration::runningDevs() {
        int cnt = 0;
        foreach (LQMeas::DevOutStream *outStream, m_outStreamDevs) {
            if (outStream->outStreamThread()->isRunning())
                cnt++;
        }
        foreach (LQMeas::DevOut *outCycle, m_outCycleDevs) {
            if (outCycle->outGenRunning())
                cnt++;
        }

        return cnt;
    }

    void OutSigGenaration::onThreadFinished() {
        LQMeas::DevOutStreamThread *thread = dynamic_cast<LQMeas::DevOutStreamThread *>(sender());
        if (thread) {            
            foreach (QSharedPointer<LQMeas::Device> dev, m_devs) {
                if (dev->devOutStream()->outStreamThread() == thread) {
                    sendError(dev->devOutStream(), thread->error());
                }
            }
        }
    }

    void OutSigGenaration::sendError(LQMeas::DevOut *outDev, LQMeas::Error err) {
        foreach (QSharedPointer<LQMeas::Device> dev, m_devs) {
            if (dev->devOut()==outDev) {
                emit deviceError(dev, err, tr("Data generation error"), tr("Error occured during data generation"));
            }
        }
    }
}
